package ca.recrown.islandsurvivalcraft.world;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.util.Random;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;
import org.junit.jupiter.api.TestInstance.Lifecycle;

import ca.recrown.islandsurvivalcraft.caching.Cache;
import ca.recrown.islandsurvivalcraft.datatypes.Point2;

@TestInstance(Lifecycle.PER_CLASS)
public class IslandWorldMapperTest {
    public final Cache<Point2, Double> blockValCache = new Cache<>(2048);
    public final int SEED = 102385923;
    public Random random = new Random(SEED);
    public IslandWorldMapper mapper = new IslandWorldMapper(random, blockValCache);
    public final double[][] answers = new double[2048][2048];

    @BeforeAll
    public void setUp() {
        for (int x = 0; x < answers.length; x++) {
            for (int y = 0; y < answers[x].length; y++) {
                answers[x][y] = mapper.getWorldBlockValue(x, y);
            }
        }
        blockValCache.clearCache();
    }

    @AfterEach
    public void individualCleanup() {
        blockValCache.clearCache();
    }

    @BeforeEach
    public void individualSetUp() {
        random = new Random(SEED);
        mapper = new IslandWorldMapper(random, blockValCache);
    }

    @Test
    public void testBlockValueConsistency() {
        for (int x = 0; x < answers.length; x++) {
            for (int y = 0; y < answers[x].length; y++) {
                assertEquals(answers[x][y], mapper.getWorldBlockValue(x, y), String.format("Occurred at (%d, %d)", x, y));
            }
        }
    }

    @Test
    public void testBlockValueConsistencyRandom() {
        for (int amount = 0; amount < 1024; amount++) {
            int x = random.nextInt(answers.length);
            int y = random.nextInt(answers[x].length);
            assertEquals(answers[x][y], mapper.getWorldBlockValue(x, y), String.format("Occurred at (%d, %d)", x, y));
        }
    }
}